/*------------------------------------------------------------------------------

                   SASEFRED Macro Library

        Name: fred_filemapxml.sas  
 Description: This macro uses the automap feature of the XMLV2
              engine to map the XML obtained from
              the fred_filegetxml macro. Otherwise, the map has already
              been manually created and specified by the user.
              The xmlmap= option on the XMLV2 engine is used to
              create the necessary datasets for reading FactSet XML.

              The SASEFRED Interface Engine                   
           Title: Mapping and Reading FRED Data using the XMLV2 engine.
     Product: SAS/ETS Software                                    
        Keys: Keys: FRED data extraction, XML FRED Data                            
       Procs: SASEFRED
       Notes: This is an uncompiled macro, used by SASEFRED to build     
              a set of SAS datasets containing FRED data via the XMLV2
              engine and the XML Mapper automapping (Thomas Cox).
              30nov2012, saskff pulled for 13.1 copy at v940m1,
                         using s0923282.
              17jun2013, saskff added prefixattributes= param
                         using s0921784.
            
              20Jul2015, saskff pulled to update 15.1 to https: instead of http:
                         using S1190360. Also added connect= option per Eric Thies suggestion.


     Support: saskff
  Parameters: automap - replace or reuse or no, when NO use access=READONLY
              mapref  - filename handle (MyMap) when automap=NO access=READONLY                   
              xmlmap  - full physical pathname and filename of the xmlmap 
              outXml  - full physical path to filename of the XML file
              dbug    - flag to get more debug info in the SAS log about the 
                        macro variables and their values. Uses PROC Printto 
                        to redirect the log information just like makefile.
                        sas macro in dvtest/testauto does.
              numIds  - integer, number that indicates the index of the time 
                        series that is passed to the macro. E.g., if 
                        idlist ='firstSeries, secondSeries', the numIds 
                        corresponding to firstSeries is 1, and the numIds 
                        corresponding to secondSeries is 2.
    prefixattributes  - flag to indicate whether or not you want the TABLE name to be
                        prepended to the SAS variable name or not. Default=Yes, but
                        for vintage dates we do not want the observations_ in front of each
                        series name so we use prefixattributes=No, since each series name 
                        is created like this: seriesname_vintagedate.
--------------------------------------------------------------------------------*/
%macro fred_filemapxml( automap=,mapref=,xmlmap=,outXml=,dbug=,numIds=,prefixattributes= );

    /*-----------------------------------------------------------------------*/
    /*-- fasayy: squotelst macro  to put ' ' around the elements of a list --*/
    /*-----------------------------------------------------------------------*/   
    %macro squotelst(strng,quote1=%str(%'),quote2=%str(%'),delim=%str( ));
        %local i squotelst;
        %let i=1;
        %do %while(%length(%qscan(&strng,&i,|)) GT 0);
            %if %length(&squotelst) EQ 0 %then %let squotelst=&quote1.%qscan(&strng,&i,|)&quote2;
            %else %let squotelst=&squotelst.&quote1.%qscan(&strng,&i,|)&quote2;
            %let i=%eval(&i + 1);
        %if %length(%qscan(&strng,&i,|)) GT 0 %then %let squotelst=&squotelst.&delim;
        %end;
        %unquote(&squotelst)
    %mend squotelst;
    
    /*--- preset status to bad for early returns ---*/
    %let fred_macro_status=1;
    %let fred_xmlout_exist=1;

    %let debug=&dbug;
    %let pats=&prefixattributes;

    %if %sysfunc(upcase(&debug))=ON %then %do;
       options mprint;
    %end;
    %else %do;
       options nonotes;        /*  s1111072, unresolved macros...*/
                               /*  %filename(newlog,newlog,tmp); */
       filename newlog "newlog.tmp";
       proc printto log=newlog new;
       run;
       options notes;
    %end;

    options validvarname=any;

    %let myxmlmap  = %str(xmlmap=);
    %let myaccess  = %str(access=READONLY);
    %let mypats    = %str(prefixattributes=);
    %let mylib     = %str(lib=outxml);
    %let myautomap = %str(automap=);

    %let nrbauto   = %upcase(&automap);
    %let nrbmapref = %str(&&mapref.&numIds);
    /* %let ext = %str(map);                                  */
    /* %let xmlmap =%sysfunc(dequote(&xmlmap));               */
    /* %let path =U:\FRED940\TEST\;                           */
    /* %let nrbxmap   = %str(&&path.&&xmlmap.&numIds..&ext);  */


    %let len = %length(&xmlmap);
    %let nrbxmap = %substr( &xmlmap, 2 , %eval(&len-2) );
    %let part1 = %substr(&nrbxmap, 1 , %eval(&len-6) );
    %let part2 = %substr(&nrbxmap, &len-4 , %eval(3) );

    %if %sysfunc(upcase(&part2))=MAP %then %do; /* the file extension is map and it is correct */
       %let nrbxmap = &part1.&numIds..&part2;
       %let nrbxmap = %squotelst(&nrbxmap);
       %put &nrbxmap;
    %end;
    %else %do; /* the file extension is not correct */
       %put ERROR: the xmlmap extension (map) need to be specified. ;
       %goto fatal_return;
    %end;

    /*--------------------------------------------------------*/
    /*--- Validation of parameters supplied to macro       ---*/
    /*--------------------------------------------------------*/
    %if &nrbxmap EQ %then %do;
        %put ERROR: XMLMAP= parameter is required for FRED_FILEMAPXML macro.;
        %goto exit;
    %end;

    /*-----------------------------------------------------------*/
    /*--- Issue XMLV2 LIBNAME assign based on automap setting ---*/
    /*--- Use the same handle name as the XML fileref......   ---*/
    /*-----------------------------------------------------------*/
    %if &nrbauto=NO %then %do;

        filename &nrbmapref "&nrbxmap";
        /* libname outxml xmlv2 xmlmap=mapref access=READONLY */
        libname outxml&numIds xmlv2 &myxmlmap.&nrbmapref &myaccess;
        proc datasets &mylib&numIds;
        run;

        %if %sysfunc(LIBREF(outxml&numIds)) ne 0 %then %do;
            %put %sysfunc(sysmsg());
            %goto fatal_return; /* return bad status since no XML file for XMLV2 engine */
        %end;

        proc contents data=outxml&numIds..OBSERVATION; run;
        proc print data=outxml&numIds..OBSERVATION; run;
        proc contents data=outxml&numIds..OBSERVATIONS; run;
        proc print data=outxml&numIds..OBSERVATIONS; run;


    %end;
    %else %do;  /* automap=YES */
        %let fred_outxml_exist = %sysfunc(FILEREF(outxml&numIds));
        %put NOTE: fred_outxml_exist flag is set to &fred_outxml_exist;

        %if %sysfunc(FILEREF(outxml&numIds)) ne 0 %then %do;
            %put %sysfunc(sysmsg());
            %goto fatal_return; /* return bad status since no XML file for XMLV2 engine */
        %end;

        filename &nrbmapref &nrbxmap;
        libname outxml&numIds xmlv2 &myxmlmap.&nrbmapref &myautomap.&nrbauto &mypats.&pats;
        proc datasets &mylib&numIds;
        run;

        %if &pats=YES %then %do; /* prefix_attributes are being used on automapper generated files, no vintages*/
      
           %if %sysfunc(LIBREF(outxml&numIds)) ne 0 %then %do;
               %put %sysfunc(sysmsg());
               %put ERROR: Empty OUTXML file. Try adding the following expanded real-time period to your SASEFRED libname statement: ;
               %put RTSTART='1776-07-04' RTEND='9999-12-31';
               %goto fatal_return; /* return bad status since no XML file for XMLV2 engine */
           %end;

           proc sql noprint;                                         
              select r.observations_count into :nobs 
              from outxml&numIds..observations AS r;                         
           quit;
        %end;
        %else %do; /* no prefix_attributes for automapper generated files, vintage dates are used then.... */
           %if %sysfunc(LIBREF(outxml&numIds)) ne 0 %then %do;
               %put %sysfunc(sysmsg());
               %put ERROR: Empty OUTXML file. Check that your vintage dates in the VINTAGE= option matches those in the URL: ;
               %put 'http://api.stlouisfed.org/fred/series/vintagedates?series_id=<your_series_id>&api_key=<your_api_key>' ;
               %put Replace your vintage date in the VINTAGE= option with one that matches.; 
               %goto fatal_return; /* return bad status since no XML file for XMLV2 engine */
           %end;

           proc sql noprint;                                         
              select r.count into :nobs 
              from outxml&numIds..observations AS r;                         
           quit;
        %end;

        %if &nobs EQ 0 %then %do;
            %put ERROR: There is no Observation data set.;
            %put ERROR: Half-baked: Observations_count is zero.;
            %goto half_baked;
        %end;

        proc contents data=outxml&numIds..OBSERVATION; run;
        proc print data=outxml&numIds..OBSERVATION; run;
 %half_baked:
        proc contents data=outxml&numIds..OBSERVATIONS; run;
        proc print data=outxml&numIds..OBSERVATIONS; run;
        
    %end;

 %exit: 
   %let fred_macro_status=0;   /* SUCCESS */

   %fatal_return:
   *-- unconditionally cut notes on --;
   options nonotes;
   %if %sysfunc(upcase(&debug))=OFF %then %do;
      proc printto log=log;
      run;

    /* -- s1111072, unresolved macros...*/
    /* -- %ridfile(newlog.tmp);         */

      %let rc = %sysfunc(fdelete(newlog));
   %end;

   options notes; options validvarname=any;

%mend fred_filemapxml;
