/* -----------------------------------------------------------------------------------------

                   SASEFRED Macro Library

        Name: fred_freqcheck_url.sas  
 Description: This macro submits a request to the FRED server and obtains the series list of 
              vintagedates, or category/series, or release/series, or tag/series... params are
              series_id= , category_id=, release_id=, tag_names= respectively. Other queries should
              work too, see https://api.stlouisfed.org/docs/fred/.
                  
       Title: Reading FRED Data by URL Access on FILENAME Statement.
     Product: SAS/ETS Software                                    
        Keys: FRED data extraction, XML FRED Data                             
       Procs: SASEFRED      
       Notes: This is an uncompiled macro, used by SASEFRED to build     
                  a FRED request for web access. An API_KEY provided by 
                  FRED is required:

                  filename trans url
                     'https://api.stlouisfed.org/fred/series/vintagedates?
                         seriesid=gnpca&api_key =0932f481a9ddeaa52778ea8706ac13ee;'

              30nov2012, saskff pulled for 13.1 copy at v940m1,
                         using s0923282.
              29aug2013, saskff pulled for 13.1 to add url= parameter, to allow different url capability
                         than the hardcoded one in the original macro named fred_freqcheck.sas,
                         using s0992131.                 
              13jun2013, saskff pulled for 14.1 to use this capability for completely designated
                         URL--most likely generates a list of series_ids or vintagedates...
              27Mar2015, saskff pulled for 14.1 v940m3 to add proxy= option for SAS/Germany 
                         using s1159906.
              20Jul2015, saskff pulled to update 15.1 to https: instead of http:
                         using S1190360. Also added connect= option per Eric Thies suggestion.

     Support: saskff

 Required Parameters: 
  urlf      - string, FRED web service urlname, such as https://api.stlouisfed.org/fred/series.
  apikeyf   - string, FRED web service required key, 
               32 character alpha-numeric lowercase string.

  automapf  - replace or reuse or no, when NO use access=READONLY.
  mapreff   - filename handle (MyMaptp) when automap=NO access=READONLY.                   
  xmlmapf   - full physical pathname and filename of the xmlmap. 
  outXmlf   - full physical path to filename of the XML file.
  memnamef  - string, membername to use for OUTTPU..&memnamef in the proc print,
              but only for the URL= option which is used when no time series observations
              are being requested. This allows the user to see information about their
              fred data from within their SAS session, to query things like sources, groups, tags.


 Optional Parameters:
  dbugf     - string, flag to get more debug info in the SAS log about the macro.
              variables and their values. Uses PROC Printto to redirect the log
              information just like makefile.sas macro in dvtest/testauto does.
  proxyf     - use a proxy server at this address
  proxyflagf - default is zero, set to one when proxyf is specified.
  connectf   - Yes or No to connect through firewall on PROXY, s1190360, hot fix.

 Not Used For Now:
  idsf      - string, BY groups or ids that the time series are organized.
  numIdsf   - integer, number that indicates the index of the time series that is
              passed to the macro. E.g., if idlist ='firstSeries, secondSeries', the 
              numIds corresponding to firstSeries is 1, and the 
              numIds corresponding to secondSeries is 2.
  freqf     - string, a lower frequency to aggregate values to, default: no value.
              for no frequency aggregation. Options: 'd','w','bw','m','q','sa',
              'a','wef','weth','wew','wetu','wem','bwew','bwem'.

  MacroStructure: 
              %macro fred_freqcheck();  Macro 1
                 %macro fred_filetransurl_f(...); Macro 1.1
                    .............
                 %mend fred_filetransurl_f; 
                    
                 %macro fred_filegetxml_f(...);   Macro 1.2
                    .............
                 %mend fred_filegetxml_f;
              
                 %macro fred_filemapxml_f(...);   Macro 1.3
                    ..............
                    %macro compareFreq;          Calling Macro 2
                 %mend fred_filemapxml_f;
                     
                 %fred_filetransurl_f;   Calling Macro 1.1
                 %fred_filegetxml_f;     Calling Macro 1.2
                 %fred_filemapxml_f;     Calling Macro 1.3
              %mend fred_freqcheck();

------------------------------------------------------------------------------------------*/


/*Macro 1 --------------------------------------------------------------------------------*/

%macro fred_freqcheck_url( urlf=,apikeyf=,outXmlf=,automapf=,mapreff=,xmlmapf=,memnamef=,
                           dbugf=,proxyf=,proxyflagf=0,connectf=OFF );
%let debug=&dbugf;

%if %sysfunc(upcase(&debug))=ON %then %do;
   options mprint symbolgen;
%end;
%else %do;
   options nonotes;        /*  s1111072, unresolved macros...*/
                           /*  %filename(newlog,newlog,tmp); */
   filename newlog "newlog.tmp";
   proc printto log=newlog new;
   run;
   options notes;
%end;

%let zero = %str(0);
%let connflag=&connectf;
%let globproxy=OFF;

%if &connflag=ON %then %do;
    %let myconnect = %str(connect);
%end;
%else %do;
    %let myconnect= ;
%end;

%if &proxyflagf EQ &zero %then %do;
   %let proxy_len=0;
%end;
%else %do;
    %let proxy_len = %eval( %length(&proxyf) );
    %let myproxy = %str(proxy=);
    
    %if &proxy_len > 2 %then %let globproxy = ON;
    
    %else %let proxy_len = 0;
%end;

%if &proxyflagf EQ &zero %then %do;
       %let proxy_len=0;
    %end;
    %else %do;
        %let proxy_len = %eval( %length(&proxyf) );
        %let myproxy = %str(proxy=);
        
        %if &proxy_len > 2 %then %let globproxy = ON;
        
        %else %let proxy_len = 0;
    %end;

/* SubMacro 1.1 --------------------------------------------------------------------------*/
%macro fred_filetransurl_furl( url=,apikey=,proxy=,proxyflag=0,connect=OFF);

    /*--- preset status to bad for early returns ---*/
    %let fred_macro_status=1;

    options validvarname=any;
 
    %let nrburl    = %str(&url);    
    %let myapi     = %nrstr(&api_key=);
   
    %let nrbapi    = %str(&apikey);
    

    %let zero = %str(0);
  
 /*--- remove the leading and trailing quotes from the url (&url=) ---*/
    %let urlnamelen = %eval( %length(&url)-2 );
   
    %if &urlnamelen <= 0 %then %do;          /*use hardcoded url instead*/
       %let nrburl =%str(https://api.stlouisfed.org/fred/series/vintagedates?seriesid=gnpca);
       %let urlnamelen = %eval( %length(&nrburl)-2 );
       %let nrburl = %substr( &nrburl, 2 , &urlnamelen );
       %goto skip0;
    %end;      
    %let nrburl = %substr( &url, 2 , &urlnamelen );
  
   %skip0:
  
 /*--- remove the leading and trailing quotes from the apikey (&api_key=) ---*/
    %let apilen = %eval( %length(&apikey)-2 );
    %if &apilen < 0 %then %goto skip1;
    %if &apilen = 0 %then %do;          /*to avoid warning message when the &apilen=0*/
       %let nrbapi =; 
       %goto skip1;
    %end;      
    %let nrbapi = %substr( &apikey, 2 , &apilen );
   
   %skip1:            
 /*--- remove the leading and trailing quotes from the time series ID (ids) (&seriesid=)--*/
    %let idlen = %eval( %length(&ids)-2 );
    %if &idlen < 0 %then %goto skip2;
    %if &idlen = 0 %then %do;       
       %let nrbid =; 
       %goto skip2;
    %end;            
    %let nrbid = %substr( &ids, 2 , &idlen );

  %skip2:
 /*--------------------------------------------------------*/
 /*--- Validation of parameters supplied to macro       ---*/
 /*--------------------------------------------------------*/  
    %if &nrbapi EQ %then %do;
        %put ERROR: API_KEY= parameter required for FRED_FILETRANSURL_FURL macro.;
        %goto exit;
    %end;

    %if &memnamef EQ %then %do;
        %put ERROR: Invalid URL= parameter, required for FRED_FILETRANSURL_FURL macro.;
        %goto exit;
    %end;
    
    /*---- strip off the 's' for the &memnam0 ----*/
    %let memlen0 = %eval( %length(&memnamef) -1 );
    %let memnam0 = %substr( &memnamef, 1, &memlen0 );

 /*--------------------------------------------------------*/
 /*--- Build FILENAME statement with URL for FRED       ---*/
 /*--------------------------------------------------------*/
    %let glob = &nrburl.&myapi.&nrbapi;
    %if &globproxy=OFF %then
       filename transtpu url "&glob"
       &myconnect;
    %else %do;
        %if &connect=ON %then %do; 
           filename transtpu url "&glob" &myproxy "&proxy"
           connect;
        %end;
        %else %do;
            filename transtpu url "&glob" &myproxy "&proxy";
        %end;
    %end;
 
%exit:
    %let fred_macro_status=0;

%mend fred_filetransurl_furl;

 
/* SubMacro 1.2 --------------------------------------------------------------------------*/

%macro fred_filegetxml_furl(outXml=,numIds=);

    /*--- preset status to bad for early returns ---*/
    %let fred_macro_status=1;
    %let fred_xmlout_exist=1;

    options validvarname=any;

    /*--------------------------------------------------------*/
    /*--- Validation of parameters supplied to macro       ---*/
    /*--------------------------------------------------------*/

    %let nrbxout = %str(&&outXml.tpu);
    %let ext = %str(xml);

    %if %eval(%length(&nrbxout) ne 0) %then %do;
       %let glob = &nrbxout..&ext;
       filename outtpu "&glob";

       data _NULL_;
       FILE outtpu;
       INFILE transtpu; /*  must match fred_filetransurl's fileref  */
       INPUT;
       PUT _infile_;
       RUN;

    %end;
    %else %do;
       filename outtpu "XFRED.xml";

       data _NULL_;
       FILE outtpu;
       INFILE transtpu; /*  must match fred_filetransurl's fileref */
       INPUT;
       PUT _infile_;
       RUN;
   
    %end;
  
    %let fred_outxml_exist = %sysfunc(FILEREF(outtpu));
    %put NOTE: fred_outxml_exist flag is set to &fred_outxml_exist;

    %if %sysfunc(FILEREF(outtpu)) ne 0 %then %do;
        %put %sysfunc(sysmsg());
        %goto bad_status; /* return bad status since no XML file for XMLV2 engine */
    %end;

    %let fred_macro_status=0;
    %bad_status:

%mend fred_filegetxml_furl;


/*SubMacro 1.3----------------------------------------------------------------------------*/

%macro fred_filemapxml_furl( automap=,mapref=,xmlmap=,outXml=, memnam=);

    /*-----------------------------------------------------------------------*/
    /*-- fasayy: squotelst macro  to put ' ' around the elements of a list --*/
    /*-----------------------------------------------------------------------*/   
    %macro squotelst(strng,quote1=%str(%'),quote2=%str(%'),delim=%str( ));
        %local i squotelst;
        %let i=1;
        %do %while(%length(%qscan(&strng,&i,|)) GT 0);
            %if %length(&squotelst) EQ 0 %then %let squotelst=&quote1.%qscan(&strng,&i,|)&quote2;
            %else %let squotelst=&squotelst.&quote1.%qscan(&strng,&i,|)&quote2;
            %let i=%eval(&i + 1);
        %if %length(%qscan(&strng,&i,|)) GT 0 %then %let squotelst=&squotelst.&delim;
        %end;
        %unquote(&squotelst)
    %mend squotelst;
    
    /*--- preset status to bad for early returns ---*/
    %let fred_macro_status=1;
    %let fred_xmlout_exist=1;
  
    %let myxmlmap  = %str(xmlmap=);
    %let myaccess  = %str(access=READONLY);
    %let mylib     = %str(lib=outtpu);
    %let myautomap = %str(automap=);

    %let nrbauto   = %upcase(&automap);
    %let nrbmapref = %str(&&mapref.tpu);

    %let len = %length(&xmlmap);
    %let nrbxmap = %substr( &xmlmap, 2 , %eval(&len-2) );
    %let part1 = %substr(&nrbxmap, 1 , %eval(&len-6) );
    %let mapName=%sysfunc(scan(&part1,-1,'\'));
    %let lenMap=%length(&mapName);
    %let part1 = %substr(&nrbxmap, 1 , %eval(&len-6-%eval(&lenMap)) );
    %let part2 = %substr(&nrbxmap, &len-4 , %eval(3) );

    %if %sysfunc(upcase(&part2))=MAP %then %do; /* the file extension is map and it is correct */
       %let nrbxmap = &part1.XFREDTPU.&part2;
       %let nrbxmap = %squotelst(&nrbxmap);
       %put &nrbxmap;
    %end;
    %else %do; /* the file extension is not correct */
       %put ERROR: the xmlmap extension (map) need to be specified. ;
       %goto fatal_return;
    %end;

    /*--------------------------------------------------------*/
    /*--- Validation of parameters supplied to macro       ---*/
    /*--------------------------------------------------------*/
    %if &nrbxmap EQ %then %do;
        %put ERROR: XMLMAP= parameter is required for FRED_FILEMAPXML macro.;
        %goto exit;
    %end;

    %let freq_match_flag=1;  /* reset the flag (failure) */

    /*-----------------------------------------------------------*/
    /*--- Issue XMLV2 LIBNAME assign based on automap setting ---*/
    /*--- Use the same handle name as the XML fileref......   ---*/
    /*-----------------------------------------------------------*/
    %if &nrbauto=NO %then %do;

        filename &nrbmapref "&nrbxmap";
        /* libname outxml xmlv2 xmlmap=mapref access=READONLY */
        libname outtpu xmlv2 &myxmlmap.&nrbmapref &myaccess;
        proc datasets &mylib;
        run;

        /* proc contents data=outtpu..SERIES; run;      */
        /* proc print data=outtpu..SERIES; run;         */
        /* proc contents data=outtpu..SERIESS; run;     */
        /* proc print data=outtpu..SERIESS; run;        */

        
    %end;
    %else %do;
        %let fred_outxml_exist = %sysfunc(FILEREF(outtpu));
        %put NOTE: fred_outxml_exist flag is set to &fred_outxml_exist;

        %if %sysfunc(FILEREF(outtpu)) ne 0 %then %do;
            %put %sysfunc(sysmsg());
            %goto fatal_return; /* return bad status since no XML file for XMLV2 engine */
        %end;

        filename &nrbmapref &nrbxmap;
        libname outtpu xmlv2 &myxmlmap.&nrbmapref &myautomap.&nrbauto;
        proc datasets &mylib;
        run;

        /*---- strip off the 's' for the &memnam ----*/
        %let memlen1 = %eval( %length(&memnam) -1 );
        %let memnam1 = %substr( &memnam, 1, &memlen1 );

        %if &memnam1=categorie %then %let memnam1 = %str(category);

        proc contents data=outtpu.&memnam1; run;  /* was outtpu.vintage_date */
        proc print data=outtpu.&memnam1; run;


        proc contents data=outtpu.&memnam; run; /* was outtpu.vintage_dates */
        proc print data=outtpu.&memnam; run;    

    %end;

 %exit:
    %let fred_macro_status=0;   /* SUCCESS */

 %fatal_return:

%mend fred_filemapxml_furl;

/* calling SubMacros 1.1, 1.2, and 1.3 */
%fred_filetransurl_furl( url=&urlf,apikey=&apikeyf,proxy=&proxyf,proxyflag=&proxyflagf,connect=&connectf);
%fred_filegetxml_furl( outXml=XFRED,numIds=1);
%fred_filemapxml_furl( automap=&automapf,mapref=&mapreff,xmlmap=&xmlmapf,outXml=&outXmlf,memnam=&memnamef);

options nonotes; 
   %if %sysfunc(upcase(&debug))=OFF %then %do;
      proc printto log=log;
      run;
    /* -- s1111072, unresolved macros...*/
    /* -- %ridfile(newlog.tmp);         */

      %let rc = %sysfunc(fdelete(newlog));
   %end; 
options notes; options validvarname=any; 

%mend fred_freqcheck_url;  /* end of Macro 1 */
